From 8c6cca0ba1424200dcb872eb96918281c9e62784 Mon Sep 17 00:00:00 2001
From: pwpiwi <pwpiwi@users.noreply.github.com>
Date: Tue, 23 Oct 2018 08:22:13 +0200
Subject: [PATCH 1/1] Fix hf 15 sim (#696)

* added ISO15693 coding for tag messages (CodeIso15693AsTag())
* added ISO15693 decoding for reader commands (Handle15693SampleFromReader())
* send tag inventory response in either high or low speed
* some refactoring and formatting
---
 armsrc/iso15693.c      | 1043 +++++++++++++++++++++++++---------------
 armsrc/util.c          |   24 +-
 client/cmdhf15.c       |  103 ++--
 common/iso15693tools.h |   64 +--
 common/protocols.h     |   60 ++-
 fpga/hi_simulate.v     |   45 +-
 6 files changed, 781 insertions(+), 558 deletions(-)

diff --git a/armsrc/iso15693.c b/armsrc/iso15693.c
index 9b0ab29e..9479c3c0 100644
--- a/armsrc/iso15693.c
+++ b/armsrc/iso15693.c
@@ -2,33 +2,31 @@
 // Jonathan Westhues, split Nov 2006
 // Modified by Greg Jones, Jan 2009
 // Modified by Adrian Dabrowski "atrox", Mar-Sept 2010,Oct 2011
+// Modified by piwi, Oct 2018 
 //
 // This code is licensed to you under the terms of the GNU GPL, version 2 or,
 // at your option, any later version. See the LICENSE.txt file for the text of
 // the license.
 //-----------------------------------------------------------------------------
 // Routines to support ISO 15693. This includes both the reader software and
-// the `fake tag' modes, but at the moment I've implemented only the reader
-// stuff, and that barely.
-// Modified to perform modulation onboard in arm rather than on PC
-// Also added additional reader commands (SELECT, READ etc.)
+// the `fake tag' modes.
 //-----------------------------------------------------------------------------
-// The ISO 15693 describes two transmission modes from reader to tag, and 4 
-// transmission modes from tag to reader. As of Mar 2010 this code only 
-// supports one of each: "1of4" mode from reader to tag, and the highspeed 
-// variant with one subcarrier from card to reader.
-// As long, as the card fully support ISO 15693 this is no problem, since the 
-// reader chooses both data rates, but some non-standard tags do not. Further for 
-// the simulation to work, we will need to support all data rates.
+
+// The ISO 15693 describes two transmission modes from reader to tag, and four
+// transmission modes from tag to reader. As of Oct 2018 this code supports
+// both reader modes and the high speed variant with one subcarrier from card to reader.
+// As long as the card fully support ISO 15693 this is no problem, since the
+// reader chooses both data rates, but some non-standard tags do not. 
+// For card simulation, the code supports both high and low speed modes with one subcarrier.
 //
 // VCD (reader) -> VICC (tag)
 // 1 out of 256:
-// 	data rate: 1,66 kbit/s (fc/8192) 
+// 	data rate: 1,66 kbit/s (fc/8192)
 // 	used for long range
 // 1 out of 4:
 // 	data rate: 26,48 kbit/s (fc/512)
 //	used for short range, high speed
-// 
+//
 // VICC (tag) -> VCD (reader)
 // Modulation:
 //		ASK / one subcarrier (423,75 khz)
@@ -39,22 +37,17 @@
 // 	high ASK: 26,48 kbit/s
 // 	high FSK: 26,69 kbit/s
 //-----------------------------------------------------------------------------
-// added "1 out of 256" mode (for VCD->PICC) - atrox 20100911
 
 
 // Random Remarks:
 // *) UID is always used "transmission order" (LSB), which is reverse to display order
 
 // TODO / BUGS / ISSUES:
-// *) writing to tags takes longer: we miss the answer from the tag in most cases
-//    -> tweak the read-timeout times
-// *) signal decoding from the card is still a bit shaky. 
-// *) signal decoding is unable to detect collissions.
-// *) add anti-collission support for inventory-commands 
+// *) signal decoding is unable to detect collisions.
+// *) add anti-collision support for inventory-commands
 // *) read security status of a block
-// *) sniffing and simulation do only support one transmission mode. need to support 
-//		all 8 transmission combinations
-//	*) remove or refactor code under "depricated"
+// *) sniffing and simulation do not support two subcarrier modes.
+// *) remove or refactor code under "depricated"
 // *) document all the functions
 
 
@@ -63,6 +56,7 @@
 #include "apps.h"
 #include "string.h"
 #include "iso15693tools.h"
+#include "protocols.h"
 #include "cmd.h"
 
 #define arraylen(x) (sizeof(x)/sizeof((x)[0]))
@@ -83,18 +77,25 @@ static int DEBUG = 0;
 #define AddCrc(data,datalen)  Iso15693AddCrc(data,datalen)
 #define sprintUID(target,uid)	Iso15693sprintUID(target,uid)
 
-// approximate amplitude=sqrt(ci^2+cq^2) 
+// approximate amplitude=sqrt(ci^2+cq^2) by amplitude = max(|ci|,|cq|) + 1/2*min(|ci|,|cq|)
 #define AMPLITUDE(ci, cq) (MAX(ABS(ci), ABS(cq)) + MIN(ABS(ci), ABS(cq))/2)
 
-// DMA buffer
-#define ISO15693_DMA_BUFFER_SIZE 128
+// buffers
+#define ISO15693_DMA_BUFFER_SIZE     128
+#define ISO15693_MAX_RESPONSE_LENGTH  36 // allows read single block with the maximum block size of 256bits. Read multiple blocks not supported yet
+#define ISO15693_MAX_COMMAND_LENGTH   45 // allows write single block with the maximum block size of 256bits. Write multiple blocks not supported yet
+
+// timing. Delays in SSP_CLK ticks.
+#define DELAY_READER_TO_ARM            8
+#define DELAY_ARM_TO_READER            1
+#define DELAY_ISO15693_VCD_TO_VICC   132 // 132/423.75kHz = 311.5us from end of EOF to start of tag response
 
 // ---------------------------
-// Signal Processing 
+// Signal Processing
 // ---------------------------
 
 // prepare data using "1 out of 4" code for later transmission
-// resulting data rate is 26,48 kbit/s (fc/512)
+// resulting data rate is 26.48 kbit/s (fc/512)
 // cmd ... data
 // n ... length of data
 static void CodeIso15693AsReader(uint8_t *cmd, int n)
@@ -174,12 +175,10 @@ static void CodeIso15693AsReader(uint8_t *cmd, int n)
 	for(i = 0; i < 4; i++) {
 		ToSendStuffBit(1);
 	}
-	
-	ToSendMax++;
 }
 
 // encode data using "1 out of 256" scheme
-// data rate is 1,66 kbit/s (fc/8192) 
+// data rate is 1,66 kbit/s (fc/8192)
 // is designed for more robust communication over longer distances
 static void CodeIso15693AsReader256(uint8_t *cmd, int n)
 {
@@ -201,7 +200,7 @@ static void CodeIso15693AsReader256(uint8_t *cmd, int n)
 	ToSendStuffBit(1);
 	ToSendStuffBit(1);
 	ToSendStuffBit(0);
-	
+
 	for(i = 0; i < n; i++) {
 		for (j = 0; j<=255; j++) {
 			if (cmd[i]==j) {
@@ -210,8 +209,8 @@ static void CodeIso15693AsReader256(uint8_t *cmd, int n)
 			} else {
 				ToSendStuffBit(1);
 				ToSendStuffBit(1);
-			}			
-		}	
+			}
+		}
 	}
 	// EOF
 	ToSendStuffBit(1);
@@ -219,10 +218,53 @@ static void CodeIso15693AsReader256(uint8_t *cmd, int n)
 	ToSendStuffBit(0);
 	ToSendStuffBit(1);
 
-	// And slack at the end, too.
-	for(i = 0; i < 24; i++) {
+	// Fill remainder of last byte with 1
+	for(i = 0; i < 4; i++) {
 		ToSendStuffBit(1);
 	}
+
+	ToSendMax++;
+}
+
+
+static void CodeIso15693AsTag(uint8_t *cmd, int n)
+{
+	ToSendReset();
+
+	// SOF
+	ToSendStuffBit(0);
+	ToSendStuffBit(0);
+	ToSendStuffBit(0);
+	ToSendStuffBit(1);
+	ToSendStuffBit(1);
+	ToSendStuffBit(1);
+	ToSendStuffBit(0);
+	ToSendStuffBit(1);
+
+	// data
+	for(int i = 0; i < n; i++) {
+		for(int j = 0; j < 8; j++) {
+			if ((cmd[i] >> j) & 0x01) {
+					ToSendStuffBit(0);
+					ToSendStuffBit(1);
+			} else {
+					ToSendStuffBit(1);
+					ToSendStuffBit(0);
+			}
+		}
+	}
+
+	// EOF
+	ToSendStuffBit(1);
+	ToSendStuffBit(0);
+	ToSendStuffBit(1);
+	ToSendStuffBit(1);
+	ToSendStuffBit(1);
+	ToSendStuffBit(0);
+	ToSendStuffBit(0);
+	ToSendStuffBit(0);
+
+	ToSendMax++;
 }
 
 
@@ -244,29 +286,50 @@ static void TransmitTo15693Tag(const uint8_t *cmd, int len)
 }
 
 //-----------------------------------------------------------------------------
-// Transmit the command (to the reader) that was placed in cmd[].
+// Transmit the tag response (to the reader) that was placed in cmd[].
 //-----------------------------------------------------------------------------
-static void TransmitTo15693Reader(const uint8_t *cmd, int len)
+static void TransmitTo15693Reader(const uint8_t *cmd, size_t len, uint32_t start_time, bool slow)
 {
+	// don't use the FPGA_HF_SIMULATOR_MODULATE_424K_8BIT minor mode. It would spoil GetCountSspClk()
 	FpgaWriteConfWord(FPGA_MAJOR_MODE_HF_SIMULATOR | FPGA_HF_SIMULATOR_MODULATE_424K);
 
+	uint8_t shift_delay = start_time & 0x00000007;
+	uint8_t bitmask = 0x00;
+	for (int i = 0; i < shift_delay; i++) {
+		bitmask |= (0x01 << i);
+	}
+
+	while (GetCountSspClk() < (start_time & 0xfffffff8)) ;
+	AT91C_BASE_SSC->SSC_THR = 0x00; // clear TXRDY
+
 	LED_C_ON();
-    for(int c = 0; c < len; ) {
-        if(AT91C_BASE_SSC->SSC_SR & (AT91C_SSC_TXRDY)) {
-            AT91C_BASE_SSC->SSC_THR = cmd[c];
-            c++;
+	uint8_t bits_to_shift = 0x00;
+    for(size_t c = 0; c <= len; c++) {
+		uint8_t bits_to_send = bits_to_shift << (8 - shift_delay) | (c==len?0x00:cmd[c]) >> shift_delay;
+		bits_to_shift = cmd[c] & bitmask;
+		for (int i = 7; i >= 0; i--) {
+			for (int j = 0; j < (slow?4:1); ) {
+				if (AT91C_BASE_SSC->SSC_SR & AT91C_SSC_TXRDY) {
+					if (bits_to_send >> i & 0x01) {
+						AT91C_BASE_SSC->SSC_THR = 0xff;
+					} else {
+						AT91C_BASE_SSC->SSC_THR = 0x00;
+					}
+					j++;
+				}
+				WDT_HIT();
+			}
         }
-        WDT_HIT();
     }
 	LED_C_OFF();
 }
 
 
 //=============================================================================
-// An ISO 15693 demodulator (one subcarrier only). Uses cross correlation to 
-// identify the SOF, each bit, and EOF.
+// An ISO 15693 decoder for tag responses (one subcarrier only).
+// Uses cross correlation to identify the SOF, each bit, and EOF.
 // This function is called 8 times per bit (every 2 subcarrier cycles).
-// Subcarrier frequency fs is 424kHz, 1/fs = 2,36us, 
+// Subcarrier frequency fs is 424kHz, 1/fs = 2,36us,
 // i.e. function is called every 4,72us
 // LED handling:
 //    LED C -> ON once we have received the SOF and are expecting the rest.
@@ -279,13 +342,13 @@ static void TransmitTo15693Reader(const uint8_t *cmd, int len)
 #define SUBCARRIER_DETECT_THRESHOLD	2
 #define SOF_CORRELATOR_LEN (1<<5)
 
-typedef struct Demod {
+typedef struct DecodeTag {
 	enum {
-		DEMOD_UNSYNCD,
-		DEMOD_AWAIT_SOF_1,
-		DEMOD_AWAIT_SOF_2,
-		DEMOD_RECEIVING_DATA,
-		DEMOD_AWAIT_EOF
+		STATE_TAG_UNSYNCD,
+		STATE_TAG_AWAIT_SOF_1,
+		STATE_TAG_AWAIT_SOF_2,
+		STATE_TAG_RECEIVING_DATA,
+		STATE_TAG_AWAIT_EOF
 	}         state;
 	int       bitCount;
 	int       posCount;
@@ -305,145 +368,145 @@ typedef struct Demod {
 	int32_t   SOF_corr;
 	int32_t   SOF_corr_prev;
 	uint8_t   SOF_correlator[SOF_CORRELATOR_LEN];
-} Demod_t;
+} DecodeTag_t;
 
-static RAMFUNC int Handle15693SamplesDemod(int8_t ci, int8_t cq, Demod_t *Demod)
+static int Handle15693SamplesFromTag(int8_t ci, int8_t cq, DecodeTag_t *DecodeTag)
 {
-	switch(Demod->state) {
-		case DEMOD_UNSYNCD:
+	switch(DecodeTag->state) {
+		case STATE_TAG_UNSYNCD:
 			// initialize SOF correlator. We are looking for 12 samples low and 12 samples high.
-			Demod->SOF_low = 0;
-			Demod->SOF_high = 12;
-			Demod->SOF_last = 23;
-			memset(Demod->SOF_correlator, 0x00, Demod->SOF_last + 1);
-			Demod->SOF_correlator[Demod->SOF_last] = AMPLITUDE(ci,cq);
-			Demod->SOF_corr = Demod->SOF_correlator[Demod->SOF_last];
-			Demod->SOF_corr_prev = Demod->SOF_corr;
-			// initialize Demodulator
-			Demod->posCount = 0;
-			Demod->bitCount = 0;
-			Demod->len = 0;
-			Demod->state = DEMOD_AWAIT_SOF_1;
+			DecodeTag->SOF_low = 0;
+			DecodeTag->SOF_high = 12;
+			DecodeTag->SOF_last = 23;
+			memset(DecodeTag->SOF_correlator, 0x00, DecodeTag->SOF_last + 1);
+			DecodeTag->SOF_correlator[DecodeTag->SOF_last] = AMPLITUDE(ci,cq);
+			DecodeTag->SOF_corr = DecodeTag->SOF_correlator[DecodeTag->SOF_last];
+			DecodeTag->SOF_corr_prev = DecodeTag->SOF_corr;
+			// initialize Decoder
+			DecodeTag->posCount = 0;
+			DecodeTag->bitCount = 0;
+			DecodeTag->len = 0;
+			DecodeTag->state = STATE_TAG_AWAIT_SOF_1;
 			break;
-			
-		case DEMOD_AWAIT_SOF_1:
+
+		case STATE_TAG_AWAIT_SOF_1:
 			// calculate the correlation in real time. Look at differences only.
-			Demod->SOF_corr += Demod->SOF_correlator[Demod->SOF_low++];
-			Demod->SOF_corr -= 2*Demod->SOF_correlator[Demod->SOF_high++];
-			Demod->SOF_last++;
-			Demod->SOF_low &= (SOF_CORRELATOR_LEN-1);
-			Demod->SOF_high &= (SOF_CORRELATOR_LEN-1);
-			Demod->SOF_last &= (SOF_CORRELATOR_LEN-1);
-			Demod->SOF_correlator[Demod->SOF_last] = AMPLITUDE(ci,cq);
-			Demod->SOF_corr += Demod->SOF_correlator[Demod->SOF_last];
+			DecodeTag->SOF_corr += DecodeTag->SOF_correlator[DecodeTag->SOF_low++];
+			DecodeTag->SOF_corr -= 2*DecodeTag->SOF_correlator[DecodeTag->SOF_high++];
+			DecodeTag->SOF_last++;
+			DecodeTag->SOF_low &= (SOF_CORRELATOR_LEN-1);
+			DecodeTag->SOF_high &= (SOF_CORRELATOR_LEN-1);
+			DecodeTag->SOF_last &= (SOF_CORRELATOR_LEN-1);
+			DecodeTag->SOF_correlator[DecodeTag->SOF_last] = AMPLITUDE(ci,cq);
+			DecodeTag->SOF_corr += DecodeTag->SOF_correlator[DecodeTag->SOF_last];
 
 			// if correlation increases for 10 consecutive samples, we are close to maximum correlation
-			if (Demod->SOF_corr > Demod->SOF_corr_prev + SUBCARRIER_DETECT_THRESHOLD) {
-				Demod->posCount++;
+			if (DecodeTag->SOF_corr > DecodeTag->SOF_corr_prev + SUBCARRIER_DETECT_THRESHOLD) {
+				DecodeTag->posCount++;
 			} else {
-				Demod->posCount = 0;
+				DecodeTag->posCount = 0;
 			}
 
-			if (Demod->posCount == 10) {	// correlation increased 10 times
-				Demod->state = DEMOD_AWAIT_SOF_2;
+			if (DecodeTag->posCount == 10) {	// correlation increased 10 times
+				DecodeTag->state = STATE_TAG_AWAIT_SOF_2;
 			}
-				
-			Demod->SOF_corr_prev = Demod->SOF_corr;
-				
+
+			DecodeTag->SOF_corr_prev = DecodeTag->SOF_corr;
+
 			break;
 
-		case DEMOD_AWAIT_SOF_2:
+		case STATE_TAG_AWAIT_SOF_2:
 			// calculate the correlation in real time. Look at differences only.
-			Demod->SOF_corr += Demod->SOF_correlator[Demod->SOF_low++];
-			Demod->SOF_corr -= 2*Demod->SOF_correlator[Demod->SOF_high++];
-			Demod->SOF_last++;
-			Demod->SOF_low &= (SOF_CORRELATOR_LEN-1);
-			Demod->SOF_high &= (SOF_CORRELATOR_LEN-1);
-			Demod->SOF_last &= (SOF_CORRELATOR_LEN-1);
-			Demod->SOF_correlator[Demod->SOF_last] = AMPLITUDE(ci,cq);
-			Demod->SOF_corr += Demod->SOF_correlator[Demod->SOF_last];
-
-			if (Demod->SOF_corr >= Demod->SOF_corr_prev) { // we are looking for the maximum correlation
-				Demod->SOF_corr_prev = Demod->SOF_corr;
+			DecodeTag->SOF_corr += DecodeTag->SOF_correlator[DecodeTag->SOF_low++];
+			DecodeTag->SOF_corr -= 2*DecodeTag->SOF_correlator[DecodeTag->SOF_high++];
+			DecodeTag->SOF_last++;
+			DecodeTag->SOF_low &= (SOF_CORRELATOR_LEN-1);
+			DecodeTag->SOF_high &= (SOF_CORRELATOR_LEN-1);
+			DecodeTag->SOF_last &= (SOF_CORRELATOR_LEN-1);
+			DecodeTag->SOF_correlator[DecodeTag->SOF_last] = AMPLITUDE(ci,cq);
+			DecodeTag->SOF_corr += DecodeTag->SOF_correlator[DecodeTag->SOF_last];
+
+			if (DecodeTag->SOF_corr >= DecodeTag->SOF_corr_prev) { // we are looking for the maximum correlation
+				DecodeTag->SOF_corr_prev = DecodeTag->SOF_corr;
 			} else {
-				Demod->lastBit = SOF_PART1;  // detected 1st part of SOF
-				Demod->sum1 = Demod->SOF_correlator[Demod->SOF_last];
-				Demod->sum2 = 0;
-				Demod->posCount = 2;
-				Demod->state = DEMOD_RECEIVING_DATA;
+				DecodeTag->lastBit = SOF_PART1;  // detected 1st part of SOF
+				DecodeTag->sum1 = DecodeTag->SOF_correlator[DecodeTag->SOF_last];
+				DecodeTag->sum2 = 0;
+				DecodeTag->posCount = 2;
+				DecodeTag->state = STATE_TAG_RECEIVING_DATA;
 				LED_C_ON();
 			}
-			
+
 			break;
 
-		case DEMOD_RECEIVING_DATA:
-			if (Demod->posCount == 1) {
-				Demod->sum1 = 0;
-				Demod->sum2 = 0;
+		case STATE_TAG_RECEIVING_DATA:
+			if (DecodeTag->posCount == 1) {
+				DecodeTag->sum1 = 0;
+				DecodeTag->sum2 = 0;
 			}
 
-			if (Demod->posCount <= 4) {
-				Demod->sum1 += AMPLITUDE(ci, cq);
+			if (DecodeTag->posCount <= 4) {
+				DecodeTag->sum1 += AMPLITUDE(ci, cq);
 			} else {
-				Demod->sum2 += AMPLITUDE(ci, cq);
+				DecodeTag->sum2 += AMPLITUDE(ci, cq);
 			}
 
-			if (Demod->posCount == 8) {
-				int16_t corr_1 = (Demod->sum2 - Demod->sum1) / 4;
-				int16_t corr_0 = (Demod->sum1 - Demod->sum2) / 4;
-				int16_t corr_EOF = (Demod->sum1 + Demod->sum2) / 8;
+			if (DecodeTag->posCount == 8) {
+				int16_t corr_1 = (DecodeTag->sum2 - DecodeTag->sum1) / 4;
+				int16_t corr_0 = (DecodeTag->sum1 - DecodeTag->sum2) / 4;
+				int16_t corr_EOF = (DecodeTag->sum1 + DecodeTag->sum2) / 8;
 				if (corr_EOF > corr_0 && corr_EOF > corr_1) {
-					Demod->state = DEMOD_AWAIT_EOF;
+					DecodeTag->state = STATE_TAG_AWAIT_EOF;
 				} else if (corr_1 > corr_0) {
 					// logic 1
-					if (Demod->lastBit == SOF_PART1) { // still part of SOF
-						Demod->lastBit = SOF_PART2;
+					if (DecodeTag->lastBit == SOF_PART1) { // still part of SOF
+						DecodeTag->lastBit = SOF_PART2;
 					} else {
-						Demod->lastBit = LOGIC1;
-						Demod->shiftReg >>= 1;
-						Demod->shiftReg |= 0x80;
-						Demod->bitCount++;
-						if (Demod->bitCount == 8) {
-							Demod->output[Demod->len] = Demod->shiftReg;
-							Demod->len++;
-							Demod->bitCount = 0;
-							Demod->shiftReg = 0;
+						DecodeTag->lastBit = LOGIC1;
+						DecodeTag->shiftReg >>= 1;
+						DecodeTag->shiftReg |= 0x80;
+						DecodeTag->bitCount++;
+						if (DecodeTag->bitCount == 8) {
+							DecodeTag->output[DecodeTag->len] = DecodeTag->shiftReg;
+							DecodeTag->len++;
+							DecodeTag->bitCount = 0;
+							DecodeTag->shiftReg = 0;
 						}
 					}
 				} else {
 					// logic 0
-					if (Demod->lastBit == SOF_PART1) { // incomplete SOF
-						Demod->state = DEMOD_UNSYNCD;
+					if (DecodeTag->lastBit == SOF_PART1) { // incomplete SOF
+						DecodeTag->state = STATE_TAG_UNSYNCD;
 						LED_C_OFF();
 					} else {
-						Demod->lastBit = LOGIC0;
-						Demod->shiftReg >>= 1;
-						Demod->bitCount++;
-						if (Demod->bitCount == 8) {
-							Demod->output[Demod->len] = Demod->shiftReg;
-							Demod->len++;
-							Demod->bitCount = 0;
-							Demod->shiftReg = 0;
+						DecodeTag->lastBit = LOGIC0;
+						DecodeTag->shiftReg >>= 1;
+						DecodeTag->bitCount++;
+						if (DecodeTag->bitCount == 8) {
+							DecodeTag->output[DecodeTag->len] = DecodeTag->shiftReg;
+							DecodeTag->len++;
+							DecodeTag->bitCount = 0;
+							DecodeTag->shiftReg = 0;
 						}
 					}
 				}
-				Demod->posCount = 0;
+				DecodeTag->posCount = 0;
 			}
-			Demod->posCount++;
+			DecodeTag->posCount++;
 			break;
-		
-		case DEMOD_AWAIT_EOF:
-			if (Demod->lastBit == LOGIC0) {  // this was already part of EOF 
+
+		case STATE_TAG_AWAIT_EOF:
+			if (DecodeTag->lastBit == LOGIC0) {  // this was already part of EOF
 				LED_C_OFF();
 				return true;
 			} else {
-				Demod->state = DEMOD_UNSYNCD;
+				DecodeTag->state = STATE_TAG_UNSYNCD;
 				LED_C_OFF();
 			}
 			break;
 
 		default:
-			Demod->state = DEMOD_UNSYNCD;
+			DecodeTag->state = STATE_TAG_UNSYNCD;
 			LED_C_OFF();
 			break;
 	}
@@ -452,15 +515,14 @@ static RAMFUNC int Handle15693SamplesDemod(int8_t ci, int8_t cq, Demod_t *Demod)
 }
 
 
-static void DemodInit(Demod_t* Demod, uint8_t* data)
+static void DecodeTagInit(DecodeTag_t *DecodeTag, uint8_t *data)
 {
-	Demod->output = data;
-	Demod->state = DEMOD_UNSYNCD;
+	DecodeTag->output = data;
+	DecodeTag->state = STATE_TAG_UNSYNCD;
 }
 
-
 /*
- *  Demodulate the samples we received from the tag, also log to tracebuffer
+ *  Receive and decode the tag response, also log to tracebuffer
  */
 static int GetIso15693AnswerFromTag(uint8_t* response, int timeout)
 {
@@ -468,19 +530,12 @@ static int GetIso15693AnswerFromTag(uint8_t* response, int timeout)
 	int lastRxCounter, samples = 0;
 	int8_t ci, cq;
 	bool gotFrame = false;
-	
-	// Allocate memory from BigBuf for some buffers
-	// free all previous allocations first
-	BigBuf_free();
 
-	// The DMA buffer, used to stream samples from the FPGA
-	uint16_t* dmaBuf = (uint16_t*) BigBuf_malloc(ISO15693_DMA_BUFFER_SIZE * sizeof(uint16_t));
+	uint16_t dmaBuf[ISO15693_DMA_BUFFER_SIZE];
 
-	// the Demodulatur data structure
-	Demod_t* Demod = (Demod_t*) BigBuf_malloc(sizeof(Demod_t));
-	
-	// Set up the demodulator for tag -> reader responses.
-	DemodInit(Demod, response);
+	// the Decoder data structure
+	DecodeTag_t DecodeTag;
+	DecodeTagInit(&DecodeTag, response);
 
 	// wait for last transfer to complete
 	while (!(AT91C_BASE_SSC->SSC_SR & AT91C_SSC_TXEMPTY));
@@ -491,7 +546,6 @@ static int GetIso15693AnswerFromTag(uint8_t* response, int timeout)
 	// Setup and start DMA.
 	FpgaSetupSsc(FPGA_MAJOR_MODE_HF_READER_RX_XCORR);
 	FpgaSetupSscDma((uint8_t*) dmaBuf, ISO15693_DMA_BUFFER_SIZE);
-
 	uint16_t *upTo = dmaBuf;
 	lastRxCounter = ISO15693_DMA_BUFFER_SIZE;
 
@@ -502,7 +556,7 @@ static int GetIso15693AnswerFromTag(uint8_t* response, int timeout)
 		}
 
 		if (behindBy < 1) continue;
-		
+
 		ci = (int8_t)(*upTo >> 8);
 		cq = (int8_t)(*upTo & 0xff);
 
@@ -518,156 +572,367 @@ static int GetIso15693AnswerFromTag(uint8_t* response, int timeout)
 		}
 		samples++;
 
-		if (Handle15693SamplesDemod(ci, cq, Demod)) {
+		if (Handle15693SamplesFromTag(ci, cq, &DecodeTag)) {
 			gotFrame = true;
 			break;
 		}
 
-		if(samples > timeout && Demod->state < DEMOD_RECEIVING_DATA) {
-			Demod->len = 0;
+		if(samples > timeout && DecodeTag.state < STATE_TAG_RECEIVING_DATA) {
+			DecodeTag.len = 0;
 			break;
 		}
+
 	}
 
 	FpgaDisableSscDma();
 
-	if (DEBUG) Dbprintf("max behindby = %d, samples = %d, gotFrame = %d, Demod.state = %d, Demod.len = %d, Demod.bitCount = %d, Demod.posCount = %d", 
-	                    maxBehindBy, samples, gotFrame, Demod->state, Demod->len, Demod->bitCount, Demod->posCount);
+	if (DEBUG) Dbprintf("max behindby = %d, samples = %d, gotFrame = %d, Decoder: state = %d, len = %d, bitCount = %d, posCount = %d",
+	                    maxBehindBy, samples, gotFrame, DecodeTag.state, DecodeTag.len, DecodeTag.bitCount, DecodeTag.posCount);
 
-	if (tracing && Demod->len > 0) {
-		uint8_t parity[MAX_PARITY_SIZE];
-		LogTrace(Demod->output, Demod->len, 0, 0, parity, false);
+	if (tracing && DecodeTag.len > 0) {
+		LogTrace(DecodeTag.output, DecodeTag.len, 0, 0, NULL, false);
 	}
 
-	return Demod->len;
+	return DecodeTag.len;
 }
 
 
-// Now the GetISO15693 message from sniffing command
-// TODO: fix it. This cannot work for several reasons: 
-//   1. Carrier is switched on during sniffing?
-//   2. We most probable miss the next reader command when demodulating
-static int GetIso15693AnswerFromSniff(uint8_t *receivedResponse, int maxLen, int *samples, int *elapsed)
+//=============================================================================
+// An ISO15693 decoder for reader commands.
+//
+// This function is called 4 times per bit (every 2 subcarrier cycles).
+// Subcarrier frequency fs is 848kHz, 1/fs = 1,18us, i.e. function is called every 2,36us
+// LED handling:
+//    LED B -> ON once we have received the SOF and are expecting the rest.
+//    LED B -> OFF once we have received EOF or are in error state or unsynced
+//
+// Returns: true  if we received a EOF
+//          false if we are still waiting for some more
+//=============================================================================
+
+typedef struct DecodeReader {
+	enum {
+		STATE_READER_UNSYNCD,
+		STATE_READER_AWAIT_1ST_RISING_EDGE_OF_SOF,
+		STATE_READER_AWAIT_2ND_FALLING_EDGE_OF_SOF,
+		STATE_READER_AWAIT_2ND_RISING_EDGE_OF_SOF,
+		STATE_READER_AWAIT_END_OF_SOF_1_OUT_OF_4,
+		STATE_READER_RECEIVE_DATA_1_OUT_OF_4,
+		STATE_READER_RECEIVE_DATA_1_OUT_OF_256
+	}           state;
+	enum {
+		CODING_1_OUT_OF_4,
+		CODING_1_OUT_OF_256
+	}           Coding;
+	uint8_t     shiftReg;
+	uint8_t     bitCount;
+	int         byteCount;
+	int         byteCountMax;
+	int         posCount;
+	int			sum1, sum2;
+	uint8_t     *output;
+} DecodeReader_t;
+
+
+static int Handle15693SampleFromReader(uint8_t bit, DecodeReader_t* DecodeReader)
 {
-	uint8_t *dest = BigBuf_get_addr();
+	switch(DecodeReader->state) {
+		case STATE_READER_UNSYNCD:
+			if(!bit) {
+				// we went low, so this could be the beginning of a SOF
+				DecodeReader->state = STATE_READER_AWAIT_1ST_RISING_EDGE_OF_SOF;
+				DecodeReader->posCount = 1;
+			}
+			break;
 
-// NOW READ RESPONSE
-	LED_D_ON();
-	FpgaWriteConfWord(FPGA_MAJOR_MODE_HF_READER_RX_XCORR);
-	//spindelay(60);	// greg - experiment to get rid of some of the 0 byte/failed reads
-	for(int c = 0; c < BIGBUF_SIZE; ) {
-		if(AT91C_BASE_SSC->SSC_SR & (AT91C_SSC_RXRDY)) {
-			uint16_t iq = AT91C_BASE_SSC->SSC_RHR;
-			// The samples are correlations against I and Q versions of the
-			// tone that the tag AM-modulates. We just want power, 
-			// so abs(I) + abs(Q) is close to what we want.
-			int8_t i = (int8_t)(iq >> 8);
-			int8_t q = (int8_t)(iq & 0xff);
-			uint8_t r = AMPLITUDE(i, q);
-			dest[c++] = r;
-		}
+		case STATE_READER_AWAIT_1ST_RISING_EDGE_OF_SOF:
+			DecodeReader->posCount++;
+			if(bit) { // detected rising edge
+				if(DecodeReader->posCount < 4) { // rising edge too early (nominally expected at 5)
+					DecodeReader->state = STATE_READER_UNSYNCD;
+				} else { // SOF
+					DecodeReader->state = STATE_READER_AWAIT_2ND_FALLING_EDGE_OF_SOF;
+				}
+			} else {
+				if(DecodeReader->posCount > 5) { // stayed low for too long
+					DecodeReader->state = STATE_READER_UNSYNCD;
+				} else {
+					// do nothing, keep waiting
+				}
+			}
+			break;
+
+		case STATE_READER_AWAIT_2ND_FALLING_EDGE_OF_SOF:
+			DecodeReader->posCount++;
+			if(!bit) { // detected a falling edge
+				if (DecodeReader->posCount < 20) {         // falling edge too early (nominally expected at 21 earliest)
+					DecodeReader->state = STATE_READER_UNSYNCD;
+				} else if (DecodeReader->posCount < 23) {  // SOF for 1 out of 4 coding
+					DecodeReader->Coding = CODING_1_OUT_OF_4;
+					DecodeReader->state = STATE_READER_AWAIT_2ND_RISING_EDGE_OF_SOF;
+				} else if (DecodeReader->posCount < 28) {  // falling edge too early (nominally expected at 29 latest)
+					DecodeReader->state = STATE_READER_UNSYNCD;
+				} else {                                 // SOF for 1 out of 4 coding
+					DecodeReader->Coding = CODING_1_OUT_OF_256;
+					DecodeReader->state = STATE_READER_AWAIT_2ND_RISING_EDGE_OF_SOF;
+				}
+			} else {
+				if(DecodeReader->posCount > 29) { // stayed high for too long
+					DecodeReader->state = STATE_READER_UNSYNCD;
+				} else {
+					// do nothing, keep waiting
+				}
+			}
+			break;
+
+		case STATE_READER_AWAIT_2ND_RISING_EDGE_OF_SOF:
+			DecodeReader->posCount++;
+			if (bit) { // detected rising edge
+				if (DecodeReader->Coding == CODING_1_OUT_OF_256) {
+					if (DecodeReader->posCount < 32) { // rising edge too early (nominally expected at 33)
+						DecodeReader->state = STATE_READER_UNSYNCD;
+					} else {
+						DecodeReader->posCount = 1;
+						DecodeReader->bitCount = 0;
+						DecodeReader->byteCount = 0;
+						DecodeReader->sum1 = 1;
+						DecodeReader->state = STATE_READER_RECEIVE_DATA_1_OUT_OF_256;
+						LED_B_ON();
+					}
+				} else { // CODING_1_OUT_OF_4
+					if (DecodeReader->posCount < 24) { // rising edge too early (nominally expected at 25)
+						DecodeReader->state = STATE_READER_UNSYNCD;
+					} else {
+						DecodeReader->state = STATE_READER_AWAIT_END_OF_SOF_1_OUT_OF_4;
+					}
+				}
+			} else {
+				if (DecodeReader->Coding == CODING_1_OUT_OF_256) {
+					if (DecodeReader->posCount > 34) { // signal stayed low for too long
+						DecodeReader->state = STATE_READER_UNSYNCD;
+					} else {
+						// do nothing, keep waiting
+					}
+				} else { // CODING_1_OUT_OF_4
+					if (DecodeReader->posCount > 26) { // signal stayed low for too long
+						DecodeReader->state = STATE_READER_UNSYNCD;
+					} else {
+						// do nothing, keep waiting
+					}
+				}
+			}
+			break;
+
+		case STATE_READER_AWAIT_END_OF_SOF_1_OUT_OF_4:
+			DecodeReader->posCount++;
+			if (bit) {
+				if (DecodeReader->posCount == 33) {
+					DecodeReader->posCount = 1;
+					DecodeReader->bitCount = 0;
+					DecodeReader->byteCount = 0;
+					DecodeReader->sum1 = 1;
+					DecodeReader->state = STATE_READER_RECEIVE_DATA_1_OUT_OF_4;
+					LED_B_ON();
+				} else {
+					// do nothing, keep waiting
+				}
+			} else { // unexpected falling edge
+				DecodeReader->state = STATE_READER_UNSYNCD;
+			}
+			break;
+
+		case STATE_READER_RECEIVE_DATA_1_OUT_OF_4:
+			DecodeReader->posCount++;
+			if (DecodeReader->posCount == 1) {
+				DecodeReader->sum1 = bit;
+			} else if (DecodeReader->posCount <= 4) {
+				DecodeReader->sum1 += bit;
+			} else if (DecodeReader->posCount == 5) {
+				DecodeReader->sum2 = bit;
+			} else {
+				DecodeReader->sum2 += bit;
+			}
+			if (DecodeReader->posCount == 8) {
+				DecodeReader->posCount = 0;
+				int corr10 = DecodeReader->sum1 - DecodeReader->sum2;
+				int corr01 = DecodeReader->sum2 - DecodeReader->sum1;
+				int corr11 = (DecodeReader->sum1 + DecodeReader->sum2) / 2;
+				if (corr01 > corr11 && corr01 > corr10) { // EOF
+					LED_B_OFF(); // Finished receiving
+					DecodeReader->state = STATE_READER_UNSYNCD;
+					if (DecodeReader->byteCount != 0) {
+						return true;
+					}
+				}
+				if (corr10 > corr11) { // detected a 2bit position
+					DecodeReader->shiftReg >>= 2;
+					DecodeReader->shiftReg |= (DecodeReader->bitCount << 6);
+				}
+				if (DecodeReader->bitCount == 15) { // we have a full byte
+					DecodeReader->output[DecodeReader->byteCount++] = DecodeReader->shiftReg;
+					if (DecodeReader->byteCount > DecodeReader->byteCountMax) {
+						// buffer overflow, give up
+						LED_B_OFF();
+						DecodeReader->state = STATE_READER_UNSYNCD;
+					}
+					DecodeReader->bitCount = 0;
+				} else {
+					DecodeReader->bitCount++;
+				}
+			}
+			break;
+
+		case STATE_READER_RECEIVE_DATA_1_OUT_OF_256:
+			DecodeReader->posCount++;
+			if (DecodeReader->posCount == 1) {
+				DecodeReader->sum1 = bit;
+			} else if (DecodeReader->posCount <= 4) {
+				DecodeReader->sum1 += bit;
+			} else if (DecodeReader->posCount == 5) {
+				DecodeReader->sum2 = bit;
+			} else {
+				DecodeReader->sum2 += bit;
+			}
+			if (DecodeReader->posCount == 8) {
+				DecodeReader->posCount = 0;
+				int corr10 = DecodeReader->sum1 - DecodeReader->sum2;
+				int corr01 = DecodeReader->sum2 - DecodeReader->sum1;
+				int corr11 = (DecodeReader->sum1 + DecodeReader->sum2) / 2;
+				if (corr01 > corr11 && corr01 > corr10) { // EOF
+					LED_B_OFF(); // Finished receiving
+					DecodeReader->state = STATE_READER_UNSYNCD;
+					if (DecodeReader->byteCount != 0) {
+						return true;
+					}
+				}
+				if (corr10 > corr11) { // detected the bit position
+					DecodeReader->shiftReg = DecodeReader->bitCount;
+				}
+				if (DecodeReader->bitCount == 255) { // we have a full byte
+					DecodeReader->output[DecodeReader->byteCount++] = DecodeReader->shiftReg;
+					if (DecodeReader->byteCount > DecodeReader->byteCountMax) {
+						// buffer overflow, give up
+						LED_B_OFF();
+						DecodeReader->state = STATE_READER_UNSYNCD;
+					}
+				}
+				DecodeReader->bitCount++;
+			}
+			break;
+
+		default:
+			LED_B_OFF();
+			DecodeReader->state = STATE_READER_UNSYNCD;
+			break;
 	}
-	FpgaWriteConfWord(FPGA_MAJOR_MODE_OFF);
+
+	return false;
+}
+
+
+static void DecodeReaderInit(uint8_t *data, uint16_t max_len, DecodeReader_t* DecodeReader)
+{
+	DecodeReader->output = data;
+	DecodeReader->byteCountMax = max_len;
+	DecodeReader->state = STATE_READER_UNSYNCD;
+	DecodeReader->byteCount = 0;
+	DecodeReader->bitCount = 0;
+	DecodeReader->shiftReg = 0;
+}
+
+
+//-----------------------------------------------------------------------------
+// Receive a command (from the reader to us, where we are the simulated tag),
+// and store it in the given buffer, up to the given maximum length. Keeps
+// spinning, waiting for a well-framed command, until either we get one
+// (returns true) or someone presses the pushbutton on the board (false).
+//
+// Assume that we're called with the SSC (to the FPGA) and ADC path set
+// correctly.
+//-----------------------------------------------------------------------------
+
+static int GetIso15693CommandFromReader(uint8_t *received, size_t max_len, uint32_t *eof_time)
+{
+	int maxBehindBy = 0;
+	int lastRxCounter, samples = 0;
+	bool gotFrame = false;
+	uint8_t b;
+
+	uint8_t dmaBuf[ISO15693_DMA_BUFFER_SIZE];
+
+	// the decoder data structure
+	DecodeReader_t DecodeReader;
+	DecodeReaderInit(received, max_len, &DecodeReader);
+
+	// wait for last transfer to complete
+	while (!(AT91C_BASE_SSC->SSC_SR & AT91C_SSC_TXEMPTY));
+
 	LED_D_OFF();
+	FpgaWriteConfWord(FPGA_MAJOR_MODE_HF_SIMULATOR | FPGA_HF_SIMULATOR_NO_MODULATION);
 
-	//////////////////////////////////////////
-	/////////// DEMODULATE ///////////////////
-	//////////////////////////////////////////
+	// clear receive register and wait for next transfer
+	uint32_t temp = AT91C_BASE_SSC->SSC_RHR;
+	(void) temp;
+	while (!(AT91C_BASE_SSC->SSC_SR & AT91C_SSC_RXRDY)) ;
 
-	int i, j;
-	int max = 0, maxPos=0;
+	uint32_t bit_time = GetCountSspClk() & 0xfffffff8;
 
-	int skip = 2;
+	// Setup and start DMA.
+	FpgaSetupSscDma(dmaBuf, ISO15693_DMA_BUFFER_SIZE);
+	uint8_t *upTo = dmaBuf;
+	lastRxCounter = ISO15693_DMA_BUFFER_SIZE;
 
-	// First, correlate for SOF
-	for(i = 0; i < 38000; i++) {
-		int corr = 0;
-		for(j = 0; j < arraylen(FrameSOF); j += skip) {
-			corr += FrameSOF[j]*dest[i+(j/skip)];
-		}
-		if(corr > max) {
-			max = corr;
-			maxPos = i;
+	for(;;) {
+		int behindBy = (lastRxCounter - AT91C_BASE_PDC_SSC->PDC_RCR) & (ISO15693_DMA_BUFFER_SIZE-1);
+		if(behindBy > maxBehindBy) {
+			maxBehindBy = behindBy;
 		}
-	}
 
-	if (DEBUG) Dbprintf("SOF at %d, correlation %d", maxPos,max/(arraylen(FrameSOF)/skip));
+		if (behindBy < 1) continue;
 
-	int k = 0; // this will be our return value
+		b = *upTo++;
+		lastRxCounter--;
+		if(upTo >= dmaBuf + ISO15693_DMA_BUFFER_SIZE) {                // we have read all of the DMA buffer content.
+			upTo = dmaBuf;                                             // start reading the circular buffer from the beginning
+			lastRxCounter += ISO15693_DMA_BUFFER_SIZE;
+		}
+		if (AT91C_BASE_SSC->SSC_SR & (AT91C_SSC_ENDRX)) {              // DMA Counter Register had reached 0, already rotated.
+			AT91C_BASE_PDC_SSC->PDC_RNPR = (uint32_t) dmaBuf;          // refresh the DMA Next Buffer and
+			AT91C_BASE_PDC_SSC->PDC_RNCR = ISO15693_DMA_BUFFER_SIZE;   // DMA Next Counter registers
+		}
 
-	// greg - If correlation is less than 1 then there's little point in continuing
-	if ((max/(arraylen(FrameSOF)/skip)) >= 1)	// THIS SHOULD BE 1
-	{
-	
-		i = maxPos + arraylen(FrameSOF)/skip;
-	
-		uint8_t outBuf[20];
-		memset(outBuf, 0, sizeof(outBuf));
-		uint8_t mask = 0x01;
-		for(;;) {
-			int corr0 = 0, corr00 = 0, corr01 = 0, corr1 = 0, corrEOF = 0;
-			for(j = 0; j < arraylen(Logic0); j += skip) {
-				corr0 += Logic0[j]*dest[i+(j/skip)];
-			}
-			corr01 = corr00 = corr0;
-			for(j = 0; j < arraylen(Logic0); j += skip) {
-				corr00 += Logic0[j]*dest[i+arraylen(Logic0)/skip+(j/skip)];
-				corr01 += Logic1[j]*dest[i+arraylen(Logic0)/skip+(j/skip)];
-			}
-			for(j = 0; j < arraylen(Logic1); j += skip) {
-				corr1 += Logic1[j]*dest[i+(j/skip)];
-			}
-			for(j = 0; j < arraylen(FrameEOF); j += skip) {
-				corrEOF += FrameEOF[j]*dest[i+(j/skip)];
-			}
-			// Even things out by the length of the target waveform.
-			corr00 *= 2;
-			corr01 *= 2;
-			corr0 *= 4;
-			corr1 *= 4;
-	
-			if(corrEOF > corr1 && corrEOF > corr00 && corrEOF > corr01) {
-				if (DEBUG) Dbprintf("EOF at %d, correlation %d (corr01: %d, corr00: %d, corr1: %d, corr0: %d)", 
-				         i, corrEOF, corr01, corr00, corr1, corr0);
-				break;
-			} else if(corr1 > corr0) {
-				i += arraylen(Logic1)/skip;
-				outBuf[k] |= mask;
-			} else {
-				i += arraylen(Logic0)/skip;
-			}
-			mask <<= 1;
-			if(mask == 0) {
-				k++;
-				mask = 0x01;
-			}
-			if((i+(int)arraylen(FrameEOF)/skip) >= BIGBUF_SIZE) {
-				DbpString("ran off end!");
+		for (int i = 7; i >= 0; i--) {
+			if (Handle15693SampleFromReader((b >> i) & 0x01, &DecodeReader)) {
+				*eof_time = bit_time + samples - DELAY_READER_TO_ARM; // end of EOF
+				gotFrame = true;
 				break;
 			}
+			samples++;
 		}
-		if(mask != 0x01) {
-			DbpString("sniff: error, uneven octet! (discard extra bits!)");
-	///		DbpString("   mask=%02x", mask);
+
+		if (gotFrame) {
+			break;
 		}
-	//	uint8_t str1 [8];
-	//	itoa(k,str1);
-	//	strncat(str1," octets read",8);
-	
-	//	DbpString(  str1);    // DbpString("%d octets", k);
-	
-	//	for(i = 0; i < k; i+=3) {
-	//		//DbpString("# %2d: %02x ", i, outBuf[i]);
-	//		DbpIntegers(outBuf[i],outBuf[i+1],outBuf[i+2]);
-	//	}
-	
-		for(i = 0; i < k; i++) {
-			receivedResponse[i] = outBuf[i];
+
+		if (BUTTON_PRESS()) {
+			DecodeReader.byteCount = 0;
+			break;
 		}
-	} // "end if correlation > 0" 	(max/(arraylen(FrameSOF)/skip))
-	return k; // return the number of bytes demodulated
 
-///	DbpString("CRC=%04x", Iso15693Crc(outBuf, k-2));
+		WDT_HIT();
+	}
+
+
+	FpgaDisableSscDma();
+
+	if (DEBUG) Dbprintf("max behindby = %d, samples = %d, gotFrame = %d, Decoder: state = %d, len = %d, bitCount = %d, posCount = %d",
+	                    maxBehindBy, samples, gotFrame, DecodeReader.state, DecodeReader.byteCount, DecodeReader.bitCount, DecodeReader.posCount);
+
+	if (tracing && DecodeReader.byteCount > 0) {
+		LogTrace(DecodeReader.output, DecodeReader.byteCount, 0, 0, NULL, true);
+	}
+
+	return DecodeReader.byteCount;
 }
 
 
@@ -681,7 +946,7 @@ void AcquireRawAdcSamplesIso15693(void)
 {
 	LEDsoff();
 	LED_A_ON();
-	
+
 	uint8_t *dest = BigBuf_get_addr();
 
 	FpgaDownloadAndGo(FPGA_BITSTREAM_HF);
@@ -718,7 +983,7 @@ void AcquireRawAdcSamplesIso15693(void)
 		if(AT91C_BASE_SSC->SSC_SR & (AT91C_SSC_RXRDY)) {
 			uint16_t iq = AT91C_BASE_SSC->SSC_RHR;
 			// The samples are correlations against I and Q versions of the
-			// tone that the tag AM-modulates. We just want power, 
+			// tone that the tag AM-modulates. We just want power,
 			// so abs(I) + abs(Q) is close to what we want.
 			int8_t i = (int8_t)(iq >> 8);
 			int8_t q = (int8_t)(iq & 0xff);
@@ -732,7 +997,7 @@ void AcquireRawAdcSamplesIso15693(void)
 }
 
 
-// TODO: there is no trigger condition. The 14000 samples represent a time frame of 66ms. 
+// TODO: there is no trigger condition. The 14000 samples represent a time frame of 66ms.
 // It is unlikely that we get something meaningful.
 // TODO: Currently we only record tag answers. Add tracing of reader commands.
 // TODO: would we get something at all? The carrier is switched on...
@@ -740,7 +1005,7 @@ void RecordRawAdcSamplesIso15693(void)
 {
 	LEDsoff();
 	LED_A_ON();
-	
+
 	uint8_t *dest = BigBuf_get_addr();
 
 	FpgaDownloadAndGo(FPGA_BITSTREAM_HF);
@@ -762,7 +1027,7 @@ void RecordRawAdcSamplesIso15693(void)
 		if(AT91C_BASE_SSC->SSC_SR & (AT91C_SSC_RXRDY)) {
 			uint16_t iq = AT91C_BASE_SSC->SSC_RHR;
 			// The samples are correlations against I and Q versions of the
-			// tone that the tag AM-modulates. We just want power, 
+			// tone that the tag AM-modulates. We just want power,
 			// so abs(I) + abs(Q) is close to what we want.
 			int8_t i = (int8_t)(iq >> 8);
 			int8_t q = (int8_t)(iq & 0xff);
@@ -778,7 +1043,7 @@ void RecordRawAdcSamplesIso15693(void)
 }
 
 
-// Initialize the proxmark as iso15k reader 
+// Initialize the proxmark as iso15k reader
 // (this might produces glitches that confuse some tags
 static void Iso15693InitReader() {
 	FpgaDownloadAndGo(FPGA_BITSTREAM_HF);
@@ -851,7 +1116,7 @@ static void BuildReadBlockRequest(uint8_t *uid, uint8_t blockNumber )
 	// Block number to read
 	cmd[10] = blockNumber;//0x00;
 	//Now the CRC
-	crc = Crc(cmd, 11); // the crc needs to be calculated over 12 bytes
+	crc = Crc(cmd, 11); // the crc needs to be calculated over 11 bytes
 	cmd[11] = crc & 0xff;
 	cmd[12] = crc >> 8;
 
@@ -860,15 +1125,13 @@ static void BuildReadBlockRequest(uint8_t *uid, uint8_t blockNumber )
 
 
 // Now the VICC>VCD responses when we are simulating a tag
-static void BuildInventoryResponse( uint8_t *uid)
+static void BuildInventoryResponse(uint8_t *uid)
 {
 	uint8_t cmd[12];
 
 	uint16_t crc;
-	// one sub-carrier, inventory, 1 slot, fast rate
-	// AFI is at bit 5 (1<<4) when doing an INVENTORY
-    //(1 << 2) | (1 << 5) | (1 << 1);
-	cmd[0] = 0; // 
+
+	cmd[0] = 0; // No error, no protocol format extension
 	cmd[1] = 0; // DSFID (data storage format identifier).  0x00 = not supported
 	// 64-bit UID
 	cmd[2] = uid[7]; //0x32;
@@ -884,21 +1147,21 @@ static void BuildInventoryResponse( uint8_t *uid)
 	cmd[10] = crc & 0xff;
 	cmd[11] = crc >> 8;
 
-	CodeIso15693AsReader(cmd, sizeof(cmd));
+	CodeIso15693AsTag(cmd, sizeof(cmd));
 }
 
 // Universal Method for sending to and recv bytes from a tag
 // 	init ... should we initialize the reader?
-// 	speed ... 0 low speed, 1 hi speed 
+// 	speed ... 0 low speed, 1 hi speed
 // 	**recv will return you a pointer to the received data
-// 	If you do not need the answer use NULL for *recv[] 
+// 	If you do not need the answer use NULL for *recv[]
 //	return: lenght of received data
 int SendDataTag(uint8_t *send, int sendlen, bool init, int speed, uint8_t **recv) {
 
 	LED_A_ON();
 	LED_B_OFF();
 	LED_C_OFF();
-	
+
 	if (init) Iso15693InitReader();
 
 	int answerLen=0;
@@ -912,22 +1175,22 @@ int SendDataTag(uint8_t *send, int sendlen, bool init, int speed, uint8_t **recv
 		// high speed (1 out of 4)
 		CodeIso15693AsReader(send, sendlen);
 	}
-	
-	TransmitTo15693Tag(ToSend,ToSendMax);	
+
+	TransmitTo15693Tag(ToSend,ToSendMax);
 	// Now wait for a response
 	if (recv!=NULL) {
-		answerLen = GetIso15693AnswerFromTag(answer, 100);	
+		answerLen = GetIso15693AnswerFromTag(answer, 100);
 		*recv=answer;
 	}
 
 	LED_A_OFF();
-	
+
 	return answerLen;
 }
 
 
 // --------------------------------------------------------------------
-// Debug Functions 
+// Debug Functions
 // --------------------------------------------------------------------
 
 // Decodes a message from a tag and displays its metadata and content
@@ -937,37 +1200,37 @@ void DbdecodeIso15693Answer(int len, uint8_t *d) {
 	uint16_t crc;
 
 	if (len>3) {
-		if (d[0]&(1<<3)) 
+		if (d[0]&(1<<3))
 			strncat(status,"ProtExt ",DBD15STATLEN);
-		if (d[0]&1) { 
+		if (d[0]&1) {
 			// error
 			strncat(status,"Error ",DBD15STATLEN);
 			switch (d[1]) {
-				case 0x01: 
+				case 0x01:
 					strncat(status,"01:notSupp",DBD15STATLEN);
 					break;
-				case 0x02: 
+				case 0x02:
 					strncat(status,"02:notRecog",DBD15STATLEN);
 					break;
-				case 0x03: 
+				case 0x03:
 					strncat(status,"03:optNotSupp",DBD15STATLEN);
 					break;
-				case 0x0f: 
+				case 0x0f:
 					strncat(status,"0f:noInfo",DBD15STATLEN);
 					break;
-				case 0x10: 
+				case 0x10:
 					strncat(status,"10:dontExist",DBD15STATLEN);
 					break;
-				case 0x11: 
+				case 0x11:
 					strncat(status,"11:lockAgain",DBD15STATLEN);
 					break;
-				case 0x12: 
+				case 0x12:
 					strncat(status,"12:locked",DBD15STATLEN);
 					break;
-				case 0x13: 
+				case 0x13:
 					strncat(status,"13:progErr",DBD15STATLEN);
 					break;
-				case 0x14: 
+				case 0x14:
 					strncat(status,"14:lockErr",DBD15STATLEN);
 					break;
 				default:
@@ -977,12 +1240,12 @@ void DbdecodeIso15693Answer(int len, uint8_t *d) {
 		} else {
 			strncat(status,"NoErr ",DBD15STATLEN);
 		}
-			
+
 		crc=Crc(d,len-2);
-		if ( (( crc & 0xff ) == d[len-2]) && (( crc >> 8 ) == d[len-1]) ) 
+		if ( (( crc & 0xff ) == d[len-2]) && (( crc >> 8 ) == d[len-1]) )
 			strncat(status,"CrcOK",DBD15STATLEN);
 		else
-			strncat(status,"CrcFail!",DBD15STATLEN); 
+			strncat(status,"CrcFail!",DBD15STATLEN);
 
 		Dbprintf("%s",status);
 	}
@@ -1035,9 +1298,9 @@ void ReaderIso15693(uint32_t parameter)
 
 	// Now send the IDENTIFY command
 	BuildIdentifyRequest();
-	
+
 	TransmitTo15693Tag(ToSend,ToSendMax);
-	
+
 	// Now wait for a response
 	answerLen1 = GetIso15693AnswerFromTag(answer1, 100) ;
 
@@ -1055,11 +1318,11 @@ void ReaderIso15693(uint32_t parameter)
 	}
 
 	Dbprintf("%d octets read from IDENTIFY request:", answerLen1);
-	DbdecodeIso15693Answer(answerLen1,answer1);
-	Dbhexdump(answerLen1,answer1,true);
+	DbdecodeIso15693Answer(answerLen1, answer1);
+	Dbhexdump(answerLen1, answer1, false);
 
 	// UID is reverse
-	if (answerLen1 >= 12) 
+	if (answerLen1 >= 12)
 		Dbprintf("UID = %02hX%02hX%02hX%02hX%02hX%02hX%02hX%02hX",
 			TagUID[7],TagUID[6],TagUID[5],TagUID[4],
 			TagUID[3],TagUID[2],TagUID[1],TagUID[0]);
@@ -1076,72 +1339,64 @@ void ReaderIso15693(uint32_t parameter)
 	// read all pages
 	if (answerLen1 >= 12 && DEBUG) {
 		uint8_t *answer2 = BigBuf_get_addr() + 4100;
-		int i=0;			
-		while (i<32) {  // sanity check, assume max 32 pages
-			BuildReadBlockRequest(TagUID,i);
-			TransmitTo15693Tag(ToSend,ToSendMax);  
+		int i = 0;
+		while (i < 32) {  // sanity check, assume max 32 pages
+			BuildReadBlockRequest(TagUID, i);
+			TransmitTo15693Tag(ToSend, ToSendMax);
 			int answerLen2 = GetIso15693AnswerFromTag(answer2, 100);
-			if (answerLen2>0) {
-				Dbprintf("READ SINGLE BLOCK %d returned %d octets:",i,answerLen2);
-				DbdecodeIso15693Answer(answerLen2,answer2);
-				Dbhexdump(answerLen2,answer2,true);
-				if ( *((uint32_t*) answer2) == 0x07160101 ) break; // exit on NoPageErr 
-			} 
+			if (answerLen2 > 0) {
+				Dbprintf("READ SINGLE BLOCK %d returned %d octets:", i, answerLen2);
+				DbdecodeIso15693Answer(answerLen2, answer2);
+				Dbhexdump(answerLen2, answer2, false);
+				if ( *((uint32_t*) answer2) == 0x07160101 ) break; // exit on NoPageErr
+			}
 			i++;
-		} 
+		}
 	}
 
-	// for the time being, switch field off to protect rdv4.0 
+	// for the time being, switch field off to protect rdv4.0
 	// note: this prevents using hf 15 cmd with s option - which isn't implemented yet anyway
    	FpgaWriteConfWord(FPGA_MAJOR_MODE_OFF);
 	LED_D_OFF();
-	
+
 	LED_A_OFF();
 }
 
-// Simulate an ISO15693 TAG, perform anti-collision and then print any reader commands
-// all demodulation performed in arm rather than host. - greg
+
+// Simulate an ISO15693 TAG.
+// For Inventory command: print command and send Inventory Response with given UID
+// TODO: interpret other reader commands and send appropriate response
 void SimTagIso15693(uint32_t parameter, uint8_t *uid)
 {
 	LEDsoff();
 	LED_A_ON();
 
-	int answerLen1 = 0;
-	int samples = 0;
-	int elapsed = 0;
-
 	FpgaDownloadAndGo(FPGA_BITSTREAM_HF);
-
-	uint8_t *buf = BigBuf_get_addr() + 4000;
-	memset(buf, 0x00, 100);
-	
 	SetAdcMuxFor(GPIO_MUXSEL_HIPKD);
-	FpgaSetupSsc(FPGA_MAJOR_MODE_HF_READER_RX_XCORR);
+   	FpgaWriteConfWord(FPGA_MAJOR_MODE_HF_SIMULATOR | FPGA_HF_SIMULATOR_NO_MODULATION);
+	FpgaSetupSsc(FPGA_MAJOR_MODE_HF_SIMULATOR);
 
-	// Start from off (no field generated)
-   	FpgaWriteConfWord(FPGA_MAJOR_MODE_OFF);
-	SpinDelay(200);
+	StartCountSspClk();
 
-	// Listen to reader
-	answerLen1 = GetIso15693AnswerFromSniff(buf, 100, &samples, &elapsed) ;
+	uint8_t cmd[ISO15693_MAX_COMMAND_LENGTH];
 
-	if (answerLen1 >=1) // we should do a better check than this
-	{
-		// Build a suitable reponse to the reader INVENTORY cocmmand
-		// not so obsvious, but in the call to BuildInventoryResponse,  the command is copied to the global ToSend buffer used below.
-		
-		BuildInventoryResponse(uid);
-	
-		TransmitTo15693Reader(ToSend,ToSendMax);
-	}
+	// Build a suitable response to the reader INVENTORY command
+	BuildInventoryResponse(uid);
 
-	Dbprintf("%d octets read from reader command: %x %x %x %x %x %x %x %x %x", answerLen1,
-		buf[0], buf[1], buf[2],	buf[3],
-		buf[4], buf[5],	buf[6], buf[7], buf[8]);
+	// Listen to reader
+	while (!BUTTON_PRESS()) {
+		uint32_t eof_time = 0, start_time = 0;
+		int cmd_len = GetIso15693CommandFromReader(cmd, sizeof(cmd), &eof_time);
+
+		if ((cmd_len >= 5) && (cmd[0] & ISO15693_REQ_INVENTORY) && (cmd[1] == ISO15693_INVENTORY)) { // TODO: check more flags
+			bool slow = !(cmd[0] & ISO15693_REQ_DATARATE_HIGH);
+			start_time = eof_time + DELAY_ISO15693_VCD_TO_VICC - DELAY_ARM_TO_READER;
+			TransmitTo15693Reader(ToSend, ToSendMax, start_time, slow);
+		}
 
-	Dbprintf("Simulationg uid: %x %x %x %x %x %x %x %x",
-		uid[0], uid[1], uid[2],	uid[3],
-		uid[4], uid[5],	uid[6], uid[7]);
+		Dbprintf("%d bytes read from reader:", cmd_len);
+		Dbhexdump(cmd_len, cmd, false);
+	}
 
 	LEDsoff();
 }
@@ -1149,50 +1404,48 @@ void SimTagIso15693(uint32_t parameter, uint8_t *uid)
 
 // Since there is no standardized way of reading the AFI out of a tag, we will brute force it
 // (some manufactures offer a way to read the AFI, though)
-void BruteforceIso15693Afi(uint32_t speed) 
-{	
+void BruteforceIso15693Afi(uint32_t speed)
+{
 	LEDsoff();
 	LED_A_ON();
-	
+
 	uint8_t data[20];
 	uint8_t *recv=data;
 	int datalen=0, recvlen=0;
-		
+
 	Iso15693InitReader();
-	
+
 	// first without AFI
-	// Tags should respond wihtout AFI and with AFI=0 even when AFI is active
-	
-	data[0]=ISO15_REQ_SUBCARRIER_SINGLE | ISO15_REQ_DATARATE_HIGH | 
-	        ISO15_REQ_INVENTORY | ISO15_REQINV_SLOT1;
-	data[1]=ISO15_CMD_INVENTORY;
-	data[2]=0; // mask length
-	datalen=AddCrc(data,3);
-	recvlen=SendDataTag(data, datalen, false, speed, &recv);
+	// Tags should respond without AFI and with AFI=0 even when AFI is active
+
+	data[0] = ISO15693_REQ_DATARATE_HIGH | ISO15693_REQ_INVENTORY | ISO15693_REQINV_SLOT1;
+	data[1] = ISO15693_INVENTORY;
+	data[2] = 0; // mask length
+	datalen = AddCrc(data,3);
+	recvlen = SendDataTag(data, datalen, false, speed, &recv);
 	WDT_HIT();
 	if (recvlen>=12) {
 		Dbprintf("NoAFI UID=%s",sprintUID(NULL,&recv[2]));
 	}
-	
+
 	// now with AFI
-	
-	data[0]=ISO15_REQ_SUBCARRIER_SINGLE | ISO15_REQ_DATARATE_HIGH | 
-	        ISO15_REQ_INVENTORY | ISO15_REQINV_AFI | ISO15_REQINV_SLOT1;
-	data[1]=ISO15_CMD_INVENTORY;
-	data[2]=0; // AFI
-	data[3]=0; // mask length
-	
+
+	data[0] = ISO15693_REQ_DATARATE_HIGH | ISO15693_REQ_INVENTORY | ISO15693_REQINV_AFI | ISO15693_REQINV_SLOT1;
+	data[1] = ISO15693_INVENTORY;
+	data[2] = 0; // AFI
+	data[3] = 0; // mask length
+
 	for (int i=0;i<256;i++) {
 		data[2]=i & 0xFF;
 		datalen=AddCrc(data,4);
 		recvlen=SendDataTag(data, datalen, false, speed, &recv);
 		WDT_HIT();
 		if (recvlen>=12) {
-			Dbprintf("AFI=%i UID=%s",i,sprintUID(NULL,&recv[2]));
+			Dbprintf("AFI=%i UID=%s", i, sprintUID(NULL,&recv[2]));
 		}
-	}	
+	}
 	Dbprintf("AFI Bruteforcing done.");
-	
+
    	FpgaWriteConfWord(FPGA_MAJOR_MODE_OFF);
 	LEDsoff();
 }
@@ -1202,31 +1455,31 @@ void DirectTag15693Command(uint32_t datalen, uint32_t speed, uint32_t recv, uint
 
 	int recvlen=0;
 	uint8_t *recvbuf = BigBuf_get_addr();
-	
+
 	LED_A_ON();
-	
+
 	if (DEBUG) {
 		Dbprintf("SEND");
-		Dbhexdump(datalen,data,true);
+		Dbhexdump(datalen, data, false);
 	}
-	
+
 	recvlen = SendDataTag(data, datalen, true, speed, (recv?&recvbuf:NULL));
 
-	if (recv) { 
+	if (recv) {
 		cmd_send(CMD_ACK, recvlen>48?48:recvlen, 0, 0, recvbuf, 48);
-		
+
 		if (DEBUG) {
 			Dbprintf("RECV");
-			DbdecodeIso15693Answer(recvlen,recvbuf); 
-			Dbhexdump(recvlen,recvbuf,true);
+			DbdecodeIso15693Answer(recvlen,recvbuf);
+			Dbhexdump(recvlen, recvbuf, false);
 		}
 	}
 
-	// for the time being, switch field off to protect rdv4.0 
+	// for the time being, switch field off to protect rdv4.0
 	// note: this prevents using hf 15 cmd with s option - which isn't implemented yet anyway
    	FpgaWriteConfWord(FPGA_MAJOR_MODE_OFF);
 	LED_D_OFF();
-	
+
 	LED_A_OFF();
 }
 
diff --git a/client/cmdhf15.c b/client/cmdhf15.c
index c116b001..e5f4af31 100644
--- a/client/cmdhf15.c
+++ b/client/cmdhf15.c
@@ -35,6 +35,7 @@
 #include "util.h"
 #include "cmdparser.h"
 #include "iso15693tools.h"
+#include "protocols.h"
 #include "cmdmain.h"
 
 #define FrameSOF              Iso15693FrameSOF
@@ -212,18 +213,17 @@ int getUID(uint8_t *buf)
 	
 	for (int retry=0;retry<3; retry++) { // don't give up the at the first try		
 		
-		req[0]= ISO15_REQ_SUBCARRIER_SINGLE | ISO15_REQ_DATARATE_HIGH | 
-		        ISO15_REQ_INVENTORY | ISO15_REQINV_SLOT1;
-		req[1]=ISO15_CMD_INVENTORY;
-		req[2]=0; // mask length
-		reqlen=AddCrc(req,3);
-		c.arg[0]=reqlen;
+		req[0] = ISO15693_REQ_DATARATE_HIGH | ISO15693_REQ_INVENTORY | ISO15693_REQINV_SLOT1;
+		req[1] = ISO15693_INVENTORY;
+		req[2] = 0; // mask length
+		reqlen = AddCrc(req,3);
+		c.arg[0] = reqlen;
 	
 		SendCommand(&c);
 		
 		if (WaitForResponseTimeout(CMD_ACK,&resp,1000)) {
 			recv = resp.d.asBytes;
-			if (resp.arg[0]>=12 && ISO15_CRC_CHECK==Crc(recv,12)) {
+			if (resp.arg[0]>=12 && ISO15693_CRC_CHECK==Crc(recv,12)) {
 			   memcpy(buf,&recv[2],8);
 			   return 1;
 			} 
@@ -424,6 +424,7 @@ int CmdHF15Sim(const char *Cmd)
 	
 	PrintAndLog("Starting simulating UID %02X %02X %02X %02X %02X %02X %02X %02X",
 			uid[0],uid[1],uid[2],uid[3],uid[4], uid[5], uid[6], uid[7]);
+	PrintAndLog("Press the button to stop simulation");
 
 	UsbCommand c = {CMD_SIMTAG_ISO_15693, {0, 0, 0}};
 	memcpy(c.d.asBytes,uid,8);
@@ -462,20 +463,19 @@ int CmdHF15DumpMem(const char*Cmd) {
 
 	for (int retry=0; retry<5; retry++) {
 		
-		req[0]= ISO15_REQ_SUBCARRIER_SINGLE | ISO15_REQ_DATARATE_HIGH | 
-		        ISO15_REQ_NONINVENTORY | ISO15_REQ_ADDRESS;
-		req[1]=ISO15_CMD_READ;
+		req[0]= ISO15693_REQ_DATARATE_HIGH | ISO15693_REQ_ADDRESS;
+		req[1] = ISO15693_READBLOCK;
 		memcpy(&req[2],uid,8);
-		req[10]=blocknum;
-		reqlen=AddCrc(req,11);
-		c.arg[0]=reqlen;
+		req[10] = blocknum;
+		reqlen = AddCrc(req,11);
+		c.arg[0] = reqlen;
 	
 		SendCommand(&c);
 		
 		if (WaitForResponseTimeout(CMD_ACK,&resp,1000)) {
 			recv = resp.d.asBytes;
-			if (ISO15_CRC_CHECK==Crc(recv,resp.arg[0])) {
-				if (!(recv[0] & ISO15_RES_ERROR)) {
+			if (ISO15693_CRC_CHECK==Crc(recv,resp.arg[0])) {
+				if (!(recv[0] & ISO15693_RES_ERROR)) {
 					retry=0;
 					*output=0; // reset outputstring
 					sprintf(output, "Block %02x   ",blocknum);
@@ -499,7 +499,7 @@ int CmdHF15DumpMem(const char*Cmd) {
   // TODO: need fix
 //	if (resp.arg[0]<3)
 //		PrintAndLog("Lost Connection");
-//	else if (ISO15_CRC_CHECK!=Crc(resp.d.asBytes,resp.arg[0]))
+//	else if (ISO15693_CRC_CHECK!=Crc(resp.d.asBytes,resp.arg[0]))
 //		PrintAndLog("CRC Failed");
 //	else 
 //		PrintAndLog("Tag returned Error %i: %s",recv[1],TagErrorStr(recv[1])); 
@@ -547,12 +547,11 @@ int CmdHF15CmdInquiry(const char *Cmd)
 	uint8_t *req=c.d.asBytes;
 	int reqlen=0;
 	
-	req[0]= ISO15_REQ_SUBCARRIER_SINGLE | ISO15_REQ_DATARATE_HIGH | 
-	        ISO15_REQ_INVENTORY | ISO15_REQINV_SLOT1;
-	req[1]=ISO15_CMD_INVENTORY;
-	req[2]=0; // mask length
+	req[0] = ISO15693_REQ_DATARATE_HIGH | ISO15693_REQ_INVENTORY | ISO15693_REQINV_SLOT1;
+	req[1] = ISO15693_INVENTORY;
+	req[2] = 0; // mask length
 	reqlen=AddCrc(req,3);
-	c.arg[0]=reqlen;
+	c.arg[0] = reqlen;
 
 	SendCommand(&c);
 	
@@ -706,7 +705,7 @@ int prepareHF15Cmd(char **cmd, UsbCommand *c, uint8_t iso15cmd[], int iso15cmdle
 	while (**cmd==' ' || **cmd=='\t') (*cmd)++;
 	
 	if (strstr(*cmd,"-o")==*cmd) {
-	 	req[reqlen]=ISO15_REQ_OPTION;
+	 	req[reqlen]=ISO15693_REQ_OPTION;
 	 	(*cmd)+=2;
 	}
 	
@@ -721,36 +720,32 @@ int prepareHF15Cmd(char **cmd, UsbCommand *c, uint8_t iso15cmd[], int iso15cmdle
 		case 's':
 		case 'S':
 			// you must have selected the tag earlier
-			req[reqlen++]|= ISO15_REQ_SUBCARRIER_SINGLE | ISO15_REQ_DATARATE_HIGH | 
-		       ISO15_REQ_NONINVENTORY | ISO15_REQ_SELECT;
-		   memcpy(&req[reqlen],&iso15cmd[0],iso15cmdlen);
-			reqlen+=iso15cmdlen;		   
+			req[reqlen++] |= ISO15693_REQ_DATARATE_HIGH | ISO15693_REQ_SELECT;
+			memcpy(&req[reqlen],&iso15cmd[0],iso15cmdlen);
+			reqlen += iso15cmdlen;		   
 		   break;
 		case 'u':
 		case 'U':
 			// unaddressed mode may not be supported by all vendors
-			req[reqlen++]|= ISO15_REQ_SUBCARRIER_SINGLE | ISO15_REQ_DATARATE_HIGH | 
-		       ISO15_REQ_NONINVENTORY;
-		   memcpy(&req[reqlen],&iso15cmd[0],iso15cmdlen);
-			reqlen+=iso15cmdlen;		   
+			req[reqlen++] |= ISO15693_REQ_DATARATE_HIGH;
+			memcpy(&req[reqlen],&iso15cmd[0],iso15cmdlen);
+			reqlen += iso15cmdlen;		   
 		   break;
 		case '*':
 			// we scan for the UID ourself
-			req[reqlen++]|= ISO15_REQ_SUBCARRIER_SINGLE | ISO15_REQ_DATARATE_HIGH | 
-		       ISO15_REQ_NONINVENTORY | ISO15_REQ_ADDRESS;
-		   memcpy(&req[reqlen],&iso15cmd[0],iso15cmdlen);
+			req[reqlen++] |= ISO15693_REQ_DATARATE_HIGH | ISO15693_REQ_ADDRESS;
+			memcpy(&req[reqlen],&iso15cmd[0],iso15cmdlen);
 			reqlen+=iso15cmdlen;		   
-		   if (!getUID(uid)) {
-		   	PrintAndLog("No Tag found");
-		   	return 0;
-		   }
-		   memcpy(req+reqlen,uid,8);
-		   PrintAndLog("Detected UID %s",sprintUID(NULL,uid));
-		   reqlen+=8;
+			if (!getUID(uid)) {
+				PrintAndLog("No Tag found");
+				return 0;
+			}
+			memcpy(req+reqlen,uid,8);
+			PrintAndLog("Detected UID %s",sprintUID(NULL,uid));
+			reqlen+=8;
 			break;			
 		default:
-			req[reqlen++]|= ISO15_REQ_SUBCARRIER_SINGLE | ISO15_REQ_DATARATE_HIGH | 
-		       ISO15_REQ_NONINVENTORY | ISO15_REQ_ADDRESS;
+			req[reqlen++] |= ISO15693_REQ_DATARATE_HIGH | ISO15693_REQ_ADDRESS;
 			memcpy(&req[reqlen],&iso15cmd[0],iso15cmdlen);
 			reqlen+=iso15cmdlen;		   
 		   
@@ -809,7 +804,7 @@ int CmdHF15CmdSysinfo(const char *Cmd) {
 		return 0;
 	}	
 	
-	prepareHF15Cmd(&cmd, &c,(uint8_t[]){ISO15_CMD_SYSINFO},1);	
+	prepareHF15Cmd(&cmd, &c,(uint8_t[]){ISO15693_GET_SYSTEM_INFO},1);	
 	reqlen=c.arg[0];
 	
 	reqlen=AddCrc(req,reqlen);
@@ -819,8 +814,8 @@ int CmdHF15CmdSysinfo(const char *Cmd) {
 
 	if (WaitForResponseTimeout(CMD_ACK,&resp,1000) && resp.arg[0]>2) {
 		recv = resp.d.asBytes;
-		if (ISO15_CRC_CHECK==Crc(recv,resp.arg[0])) {
-			if (!(recv[0] & ISO15_RES_ERROR)) {
+		if (ISO15693_CRC_CHECK==Crc(recv,resp.arg[0])) {
+			if (!(recv[0] & ISO15693_RES_ERROR)) {
 				*output=0; // reset outputstring
 				for ( i=1; i<resp.arg[0]-2; i++) {
 					sprintf(output+strlen(output),"%02X ",recv[i]);
@@ -896,7 +891,7 @@ int CmdHF15CmdReadmulti(const char *Cmd) {
 		return 0;
 	}	
 	
-	prepareHF15Cmd(&cmd, &c,(uint8_t[]){ISO15_CMD_READMULTI},1);	
+	prepareHF15Cmd(&cmd, &c,(uint8_t[]){ISO15693_READ_MULTI_BLOCK},1);	
 	reqlen=c.arg[0];
 
 	pagenum=strtol(cmd,NULL,0);
@@ -920,8 +915,8 @@ int CmdHF15CmdReadmulti(const char *Cmd) {
 
 	if (WaitForResponseTimeout(CMD_ACK,&resp,1000) && resp.arg[0]>2) {
 		recv = resp.d.asBytes;
-		if (ISO15_CRC_CHECK==Crc(recv,resp.arg[0])) {
-			if (!(recv[0] & ISO15_RES_ERROR)) {
+		if (ISO15693_CRC_CHECK==Crc(recv,resp.arg[0])) {
+			if (!(recv[0] & ISO15693_RES_ERROR)) {
 				*output=0; // reset outputstring
 				for ( int i=1; i<resp.arg[0]-2; i++) {
 					sprintf(output+strlen(output),"%02X ",recv[i]);
@@ -974,7 +969,7 @@ int CmdHF15CmdRead(const char *Cmd) {
 		return 0;
 	}	
 	
-	prepareHF15Cmd(&cmd, &c,(uint8_t[]){ISO15_CMD_READ},1);	
+	prepareHF15Cmd(&cmd, &c,(uint8_t[]){ISO15693_READBLOCK},1);	
 	reqlen=c.arg[0];
 
 	pagenum=strtol(cmd,NULL,0);
@@ -993,8 +988,8 @@ int CmdHF15CmdRead(const char *Cmd) {
 
 	if (WaitForResponseTimeout(CMD_ACK,&resp,1000) && resp.arg[0]>2) {
 		recv = resp.d.asBytes;
-		if (ISO15_CRC_CHECK==Crc(recv,resp.arg[0])) {
-			if (!(recv[0] & ISO15_RES_ERROR)) {
+		if (ISO15693_CRC_CHECK==Crc(recv,resp.arg[0])) {
+			if (!(recv[0] & ISO15693_RES_ERROR)) {
 				*output=0; // reset outputstring
 				//sprintf(output, "Block %2i   ",blocknum);
 				for ( int i=1; i<resp.arg[0]-2; i++) {
@@ -1051,7 +1046,7 @@ int CmdHF15CmdWrite(const char *Cmd) {
 		return 0;
 	}	
 	
-	prepareHF15Cmd(&cmd, &c,(uint8_t[]){ISO15_CMD_WRITE},1);	
+	prepareHF15Cmd(&cmd, &c,(uint8_t[]){ISO15693_WRITEBLOCK},1);	
 	reqlen=c.arg[0];
 	
 	// *cmd -> page num ; *cmd2 -> data 
@@ -1086,8 +1081,8 @@ int CmdHF15CmdWrite(const char *Cmd) {
 
 	if (WaitForResponseTimeout(CMD_ACK,&resp,2000) && resp.arg[0]>2) {
 		recv = resp.d.asBytes;
-		if (ISO15_CRC_CHECK==Crc(recv,resp.arg[0])) {
-			if (!(recv[0] & ISO15_RES_ERROR)) {					
+		if (ISO15693_CRC_CHECK==Crc(recv,resp.arg[0])) {
+			if (!(recv[0] & ISO15693_RES_ERROR)) {					
 				PrintAndLog("OK");	
 			} else {
 				PrintAndLog("Tag returned Error %i: %s",recv[1],TagErrorStr(recv[1])); 
diff --git a/common/iso15693tools.h b/common/iso15693tools.h
index ec63728f..96095fba 100644
--- a/common/iso15693tools.h
+++ b/common/iso15693tools.h
@@ -5,67 +5,9 @@
 #define ISO15693_H__
 
 // ISO15693 CRC
-#define ISO15_CRC_PRESET	(uint16_t)0xFFFF
-#define ISO15_CRC_POLY		(uint16_t)0x8408
-#define ISO15_CRC_CHECK		((uint16_t)(~0xF0B8 & 0xFFFF)) 	// use this for checking of a correct crc
-
-// REQUEST FLAGS
-
-#define ISO15_REQ_SUBCARRIER_SINGLE	0x00	// Tag should respond using one subcarrier (ASK)
-#define ISO15_REQ_SUBCARRIER_TWO		0x01	// Tag should respond using two subcarriers (FSK)
-#define ISO15_REQ_DATARATE_LOW		0x00	// Tag should respond using low data rate
-#define ISO15_REQ_DATARATE_HIGH		0x02	// Tag should respond using high data rate
-#define ISO15_REQ_NONINVENTORY		0x00
-#define ISO15_REQ_INVENTORY		0x04	// This is an inventory request - see inventory flags
-#define ISO15_REQ_PROTOCOL_NONEXT		0x00
-#define ISO15_REQ_PROTOCOL_EXT		0x08	// RFU
-
-// REQUEST FLAGS when INVENTORY is not set
-
-#define ISO15_REQ_SELECT			0x10	// only selected cards response
-#define ISO15_REQ_ADDRESS			0x20	// this req contains an address
-#define ISO15_REQ_OPTION			0x40	// Command specific option selector
-
-//REQUEST FLAGS when INVENTORY is set
-
-#define ISO15_REQINV_AFI			0x10	// AFI Field is present
-#define ISO15_REQINV_SLOT1		0x20	// 1 Slot
-#define ISO15_REQINV_SLOT16		0x00	// 16 Slots
-#define ISO15_REQINV_OPTION		0x40	// Command specific option selector
-
-//RESPONSE FLAGS
-#define ISO15_RES_ERROR			0x01
-#define ISO15_RES_EXT			0x08	// Protocol Extention	
-
-// RESPONSE ERROR CODES
-#define ISO15_NOERROR			0x00
-#define ISO15_ERROR_CMD_NOT_SUP		0x01	// Command not supported
-#define ISO15_ERROR_CMD_NOT_REC		0x02	// Command not recognized (eg. parameter error)
-#define ISO15_ERROR_CMD_OPTION		0x03	// Command option not supported
-#define ISO15_ERROR_GENERIC		0x0F	// No additional Info about this error
-#define ISO15_ERROR_BLOCK_UNAVAILABLE	0x10	
-#define ISO15_ERROR_BLOCK_LOCKED_ALREADY	0x11	// cannot lock again
-#define ISO15_ERROR_BLOCK_LOCKED		0x12	// cannot be changed
-#define ISO15_ERROR_BLOCK_WRITE		0x13	// Writing was unsuccessful
-#define ISO15_ERROR_BLOCL_WRITELOCK	0x14	// Locking was unsuccessful
-
-// COMMAND CODES
-#define ISO15_CMD_INVENTORY			0x01
-#define ISO15_CMD_STAYQUIET			0x02
-#define ISO15_CMD_READ					0x20
-#define ISO15_CMD_WRITE					0x21
-#define ISO15_CMD_LOCK					0x22
-#define ISO15_CMD_READMULTI			0x23
-#define ISO15_CMD_WRITEMULTI			0x24
-#define ISO15_CMD_SELECT				0x25
-#define ISO15_CMD_RESET					0x26
-#define ISO15_CMD_WRITEAFI				0x27
-#define ISO15_CMD_LOCKAFI				0x28
-#define ISO15_CMD_WRITEDSFID			0x29
-#define ISO15_CMD_LOCKDSFID			0x2A
-#define ISO15_CMD_SYSINFO				0x2B
-#define ISO15_CMD_SECSTATUS			0x2C
-
+#define ISO15693_CRC_PRESET  (uint16_t)0xFFFF
+#define ISO15693_CRC_POLY    (uint16_t)0x8408
+#define ISO15693_CRC_CHECK   ((uint16_t)(~0xF0B8 & 0xFFFF))  // use this for checking of a correct crc
 
 uint16_t Iso15693Crc(uint8_t *v, int n);
 int Iso15693AddCrc(uint8_t *req, int n);
diff --git a/common/protocols.h b/common/protocols.h
index 9ba69d5c..06a80de1 100644
--- a/common/protocols.h
+++ b/common/protocols.h
@@ -167,22 +167,50 @@ NXP/Philips CUSTOM COMMANDS
 #define ISO14443B_COMPLETION   0x0F
 #define ISO14443B_AUTHENTICATE 0x0A
 
-//First byte is 26
-#define ISO15693_INVENTORY     0x01
-#define ISO15693_STAYQUIET     0x02
-//First byte is 02
-#define ISO15693_READBLOCK            0x20
-#define ISO15693_WRITEBLOCK           0x21
-#define ISO15693_LOCKBLOCK            0x22
-#define ISO15693_READ_MULTI_BLOCK     0x23
-#define ISO15693_SELECT               0x25
-#define ISO15693_RESET_TO_READY       0x26
-#define ISO15693_WRITE_AFI            0x27
-#define ISO15693_LOCK_AFI             0x28
-#define ISO15693_WRITE_DSFID          0x29
-#define ISO15693_LOCK_DSFID           0x2A
-#define ISO15693_GET_SYSTEM_INFO      0x2B
-#define ISO15693_READ_MULTI_SECSTATUS 0x2C
+// ISO15693 COMMANDS
+#define ISO15693_INVENTORY                   0x01
+#define ISO15693_STAYQUIET                   0x02
+#define ISO15693_READBLOCK                   0x20
+#define ISO15693_WRITEBLOCK                  0x21
+#define ISO15693_LOCKBLOCK                   0x22
+#define ISO15693_READ_MULTI_BLOCK            0x23
+#define ISO15693_SELECT                      0x25
+#define ISO15693_RESET_TO_READY              0x26
+#define ISO15693_WRITE_AFI                   0x27
+#define ISO15693_LOCK_AFI                    0x28
+#define ISO15693_WRITE_DSFID                 0x29
+#define ISO15693_LOCK_DSFID                  0x2A
+#define ISO15693_GET_SYSTEM_INFO             0x2B
+#define ISO15693_READ_MULTI_SECSTATUS        0x2C
+
+// ISO15693 REQUEST FLAGS
+#define ISO15693_REQ_SUBCARRIER_TWO          (1<<0)
+#define ISO15693_REQ_DATARATE_HIGH           (1<<1)
+#define ISO15693_REQ_INVENTORY               (1<<2)
+#define ISO15693_REQ_PROTOCOL_EXT            (1<<3) // RFU
+#define ISO15693_REQ_OPTION                  (1<<6) // Command specific option selector
+// when REQ_INVENTORY is not set
+#define ISO15693_REQ_SELECT                  (1<<4) // only selected cards response
+#define ISO15693_REQ_ADDRESS                 (1<<5) // this req contains an address
+// when REQ_INVENTORY is set
+#define ISO15693_REQINV_AFI                  (1<<4) // AFI Field is present
+#define ISO15693_REQINV_SLOT1                (1<<5) // 1 Slot     (16 slots if not set)
+
+// ISO15693 RESPONSE FLAGS
+#define ISO15693_RES_ERROR                   (1<<0)
+#define ISO15693_RES_EXT                     (1<<3) // Protocol Extention	
+
+// ISO15693 RESPONSE ERROR CODES
+#define ISO15693_NOERROR                     0x00
+#define ISO15693_ERROR_CMD_NOT_SUP           0x01 // Command not supported
+#define ISO15693_ERROR_CMD_NOT_REC           0x02 // Command not recognized (eg. parameter error)
+#define ISO15693_ERROR_CMD_OPTION            0x03 // Command option not supported
+#define ISO15693_ERROR_GENERIC               0x0F // No additional Info about this error
+#define ISO15693_ERROR_BLOCK_UNAVAILABLE     0x10
+#define ISO15693_ERROR_BLOCK_LOCKED_ALREADY  0x11 // cannot lock again
+#define ISO15693_ERROR_BLOCK_LOCKED          0x12 // cannot be changed
+#define ISO15693_ERROR_BLOCK_WRITE           0x13 // Writing was unsuccessful
+#define ISO15693_ERROR_BLOCL_WRITELOCK       0x14 // Locking was unsuccessful
 
 
 // Topaz command set:
diff --git a/armsrc/util.c b/armsrc/util.c
index a1b0f151..fbb6d489 100644
--- a/armsrc/util.c
+++ b/armsrc/util.c
@@ -373,13 +373,13 @@ void StartCountSspClk()
 	AT91C_BASE_TC1->TC_CMR = AT91C_TC_CLKS_TIMER_DIV1_CLOCK // TC1 Clock = MCK(48MHz)/2 = 24MHz
 							| AT91C_TC_CPCSTOP				// Stop clock on RC compare
 							| AT91C_TC_EEVTEDG_RISING		// Trigger on rising edge of Event
-							| AT91C_TC_EEVT_TIOB			// Event-Source: TIOB1 (= ssp_clk from FPGA = 13,56MHz/16)
+							| AT91C_TC_EEVT_TIOB			// Event-Source: TIOB1 (= ssp_clk from FPGA = 13,56MHz/16 ... 13,56MHz/4)
 							| AT91C_TC_ENETRG				// Enable external trigger event
 							| AT91C_TC_WAVESEL_UP	 		// Upmode without automatic trigger on RC compare
 							| AT91C_TC_WAVE 				// Waveform Mode
 							| AT91C_TC_AEEVT_SET 			// Set TIOA1 on external event
 							| AT91C_TC_ACPC_CLEAR; 			// Clear TIOA1 on RC Compare
-	AT91C_BASE_TC1->TC_RC = 0x04; 							// RC Compare value = 0x04
+	AT91C_BASE_TC1->TC_RC = 0x02; 							// RC Compare value = 0x02
 
 	// use TC0 to count TIOA1 pulses
 	AT91C_BASE_TC0->TC_CCR = AT91C_TC_CLKDIS;				// disable TC0
@@ -402,7 +402,7 @@ void StartCountSspClk()
 	AT91C_BASE_TC2->TC_CCR = AT91C_TC_CLKEN;				// enable TC2
 
 	//
-	// synchronize the counter with the ssp_frame signal. Note: FPGA must be in any iso14443 mode, otherwise SSC_FRAME and SSC_CLK signals would not be present 
+	// synchronize the counter with the ssp_frame signal. Note: FPGA must be in a FPGA mode with SSC transfer, otherwise SSC_FRAME and SSC_CLK signals would not be present 
 	//
 	while(!(AT91C_BASE_PIOA->PIO_PDSR & GPIO_SSC_FRAME)); 	// wait for ssp_frame to go high (start of frame)
 	while(AT91C_BASE_PIOA->PIO_PDSR & GPIO_SSC_FRAME); 		// wait for ssp_frame to be low
@@ -430,15 +430,15 @@ void ResetSspClk(void) {
 }
 
 
-uint32_t RAMFUNC GetCountSspClk(){
-	uint32_t tmp_count;
-	tmp_count = (AT91C_BASE_TC2->TC_CV << 16) | AT91C_BASE_TC0->TC_CV;
-	if ((tmp_count & 0x0000ffff) == 0) { //small chance that we may have missed an increment in TC2
-		return (AT91C_BASE_TC2->TC_CV << 16);
-	} 
-	else {
-		return tmp_count;
-	}
+uint32_t GetCountSspClk(){
+	uint32_t hi, lo;
+
+	do {
+		hi = AT91C_BASE_TC2->TC_CV;
+		lo = AT91C_BASE_TC0->TC_CV;
+	} while(hi != AT91C_BASE_TC2->TC_CV);
+
+	return (hi << 16) | lo;
 }
 
 
diff --git a/fpga/hi_simulate.v b/fpga/hi_simulate.v
index 78650c4a..8d70bb1b 100644
--- a/fpga/hi_simulate.v
+++ b/fpga/hi_simulate.v
@@ -16,6 +16,13 @@
 // Jonathan Westhues, October 2006
 //-----------------------------------------------------------------------------
 
+// possible mod_types:
+`define NO_MODULATION        3'b000
+`define MODULATE_BPSK        3'b001
+`define MODULATE_212K        3'b010
+`define MODULATE_424K        3'b100
+`define MODULATE_424K_8BIT   3'b101
+
 module hi_simulate(
     pck0, ck_1356meg, ck_1356megb,
     pwr_lo, pwr_hi, pwr_oe1, pwr_oe2, pwr_oe3, pwr_oe4,
@@ -35,10 +42,6 @@ module hi_simulate(
     output dbg;
     input [2:0] mod_type;
 
-// Power amp goes between LOW and tri-state, so pwr_hi (and pwr_lo) can
-// always be low.
-assign pwr_hi = 1'b0;
-assign pwr_lo = 1'b0;
 
 // The comparator with hysteresis on the output from the peak detector.
 reg after_hysteresis;
@@ -52,8 +55,8 @@ end
 
 
 // Divide 13.56 MHz to produce various frequencies for SSP_CLK
-// and modulation. 11 bits allow for factors of up to /128.
-reg [10:0] ssp_clk_divider;
+// and modulation.
+reg [7:0] ssp_clk_divider;
 
 always @(posedge adc_clk)
     ssp_clk_divider <= (ssp_clk_divider + 1);
@@ -62,10 +65,10 @@ reg ssp_clk;
 
 always @(negedge adc_clk)
 begin
-    if(mod_type == 3'b101)
+    if(mod_type == `MODULATE_424K_8BIT)
       // Get bit every at 53KHz (every 8th carrier bit of 424kHz)
       ssp_clk <= ssp_clk_divider[7];
-    else if(mod_type == 3'b010)
+    else if(mod_type == `MODULATE_212K)
       // Get next bit at 212kHz
       ssp_clk <= ssp_clk_divider[5];
     else
@@ -89,7 +92,7 @@ always @(negedge ssp_clk)
 
 reg ssp_frame;
 always @(ssp_frame_divider_to_arm or ssp_frame_divider_from_arm or mod_type)
-    if(mod_type == 3'b000) // not modulating, so listening, to ARM
+    if(mod_type == `NO_MODULATION) // not modulating, so listening, to ARM
         ssp_frame = (ssp_frame_divider_to_arm == 3'b000);
     else
         ssp_frame = (ssp_frame_divider_from_arm == 3'b000);
@@ -102,27 +105,29 @@ always @(posedge ssp_clk)
 // Modulating carrier frequency is fc/64 (212kHz) to fc/16 (848kHz). Reuse ssp_clk divider for that.
 reg modulating_carrier;
 always @(mod_type or ssp_clk or ssp_dout)
-    if(mod_type == 3'b000)
+    if (mod_type == `NO_MODULATION)
         modulating_carrier <= 1'b0;                          // no modulation
-    else if(mod_type == 3'b001)
+    else if (mod_type == `MODULATE_BPSK)
         modulating_carrier <= ssp_dout ^ ssp_clk_divider[3]; // XOR means BPSK
-    else if(mod_type == 3'b010)
+    else if (mod_type == `MODULATE_212K)
         modulating_carrier <= ssp_dout & ssp_clk_divider[5]; // switch 212kHz subcarrier on/off
-    else if(mod_type == 3'b100 || mod_type == 3'b101)
+    else if (mod_type == `MODULATE_424K || mod_type == `MODULATE_424K_8BIT)
         modulating_carrier <= ssp_dout & ssp_clk_divider[4]; // switch 424kHz modulation on/off
     else
         modulating_carrier <= 1'b0;                           // yet unused
 
-// This one is all LF, so doesn't matter
-assign pwr_oe2 = modulating_carrier;
 
 // Toggle only one of these, since we are already producing much deeper
 // modulation than a real tag would.
-assign pwr_oe1 = modulating_carrier;
-assign pwr_oe4 = modulating_carrier;
+assign pwr_hi = 1'b0;                   // HF antenna connected to GND
+assign pwr_oe3 = 1'b0;                  // 10k Load
+assign pwr_oe1 = modulating_carrier;    // 33 Ohms Load
+assign pwr_oe4 = modulating_carrier;    // 33 Ohms Load
+
+// This is all LF and doesn't matter
+assign pwr_lo = 1'b0;
+assign pwr_oe2 = 1'b0;
 
-// This one is always on, so that we can watch the carrier.
-assign pwr_oe3 = 1'b0;
 
 assign dbg = ssp_din;
 
@@ -102,27 +105,29 @@ always @(posedge ssp_clk)
 
-// Toggle only one of these, since we are already producing much deeper
+// Load modulation. Toggle only one of these, since we are already producing much deeper
 // modulation than a real tag would.
-- 
2.39.5